    % ----------------------------------------------------------------
    % function saliency_UFO
    % input:  I ......Input image
    % output: UFO.....Output Saliency Map
    % This function combines three components to generate the UFO saliency map.
    % ----------------------------------------------------------------
    
    function [UFO Uniqueness Focusness Objectness] = saliency_UFO( I )
           
        [ hsum wsum I] = ImageCrop( I );
        ObjectnessMap  = CalObjectness( I );
        ObjectnessMap = mat2gray(ObjectnessMap);
        
        %set the minimum region area parameter
        MinimumRegionArea=uint16(sum(sum(im2bw(ObjectnessMap,graythresh(ObjectnessMap))))/10);
        %segmentation by Mean-Shift       
        [fimg labels modes regsize grad conf] = edison_wrapper(I,@RGB2Luv,'SpatialBandWidth',20,'RangeBandWidth',15,'MinimumRegionArea',MinimumRegionArea);%segment the image to regions and compute the GradMap.
        clusterNumber=max(max(labels));

        Igray = rgb2gray(I);
        [minH minW] = size(Igray);
        
        %detect the boundary for each region
        op = [1 0 -1];
        x_mask = op';
        y_mask = op;
        edgemapx=filter2(x_mask,uint8(labels),'same');
        edgemapy=filter2(y_mask,uint8(labels),'same');
        edgemap=edgemapx|edgemapy;
        edgemap(1,:)=1;
        edgemap(minH,:)=1;
        edgemap(:,1)=1;
        edgemap(:,minW)=1;
        %all edge pixels (boundary pixels and interior edge pixels)
       [CannyMap CannyThresh] = edge(Igray,'canny');
        CannyMap=CannyMap|edgemap;

        %GradMap and ScaleMap
        GradMap=grad;
        ScaleMap=CalScale(Igray);
        
        maxgrad=max(max(grad));
        GradMap(1,:)=-maxgrad;
        GradMap(minH,:)=-maxgrad;
        GradMap(:,1)=-maxgrad;
        GradMap(:,minW)=-maxgrad;      
        
        %computer mean Objectness, Grad and Scale for each region
        RegionGrad=zeros(minH,minW);
        RegionScale=zeros(minH,minW);
        Objectness=zeros(minH,minW);
        
        for k = 0:clusterNumber
            
            RegionGrad(labels==k)=sum(sum(GradMap.*edgemap.*(labels==k)))/double(sum(sum(edgemap.*(labels==k))));
            RegionScale(labels==k)=sum(sum((CannyMap.*(labels==k)).*(1./ScaleMap)))/double(sum(sum(CannyMap.*(labels==k))));
            Objectness(labels==k)=sum(sum((labels==k).*ObjectnessMap))/double(regsize(k+1));
            
        end
        
        Objectness=mat2gray(Objectness);
        RegionScale=mat2gray(RegionScale);
        RegionGrad=mat2gray(RegionGrad);   
        
        Uniqueness=CalUniqueness(I, labels, clusterNumber);
        Focusness=mat2gray((RegionGrad).*exp(RegionScale));         
        UFO=mat2gray(exp(Focusness+Uniqueness).*Objectness);
        
        %resize to original size.
        UFO = padarray(UFO,[hsum wsum],'replicate','both');
        UFO=uint8(UFO*255); 
        Uniqueness=padarray(mat2gray(Uniqueness),[hsum wsum],'replicate','both');
        Uniqueness=uint8(Uniqueness*255);
        Focusness = padarray(Focusness,[hsum wsum],'replicate','both');
        Focusness=uint8(Focusness*255); 
        Objectness = padarray(Objectness,[hsum wsum],'replicate','both');
        Objectness=uint8(Objectness*255); 
        
    end

